<?php

namespace Drupal\lupus_ce_renderer;

use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Controller\TitleResolverInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Render\MainContent\MainContentRendererInterface;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Renders main content only.
 */
class CustomElementsRenderer implements MainContentRendererInterface {

  /**
   * The title resolver.
   *
   * @var \Drupal\Core\Controller\TitleResolverInterface
   */
  protected $titleResolver;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The renderer configuration array.
   *
   * @var array
   * @see sites/default/default.services.yml
   */
  protected $rendererConfig;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs the renderer.
   *
   * @param \Drupal\Core\Controller\TitleResolverInterface $title_resolver
   *   The title resolver.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param array $renderer_config
   *   The renderer configuration array.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   */
  public function __construct(TitleResolverInterface $title_resolver, RendererInterface $renderer, array $renderer_config, ModuleHandlerInterface $module_handler) {
    $this->titleResolver = $title_resolver;
    $this->renderer = $renderer;
    $this->rendererConfig = $renderer_config;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public function renderResponse(array $main_content, Request $request, RouteMatchInterface $route_match) {
    // First render the main content, because it might provide a title.
    $render_context = new RenderContext();
    $content = $this->renderer->executeInRenderContext($render_context, function () use (&$main_content) {
      return $this->renderer->render($main_content, TRUE);
    });
    // RendererInterface::render() always causes bubbleable metadata to be
    // stored in the render context, no need to check it conditionally.
    // We need it to forward caching information.
    // @see HtmlRenderer
    /** @var \Drupal\Core\Render\BubbleableMetadata $bubbleable_metadata */
    $bubbleable_metadata = $render_context->pop();
    $bubbleable_metadata->addCacheContexts($this->rendererConfig['required_cache_contexts']);

    // Determine the title: use the title provided by the main content if any,
    // otherwise get it from the routing information.
    $title = isset($main_content['#title']) ? $main_content['#title'] : $this->titleResolver->getTitle($request, $route_match->getRouteObject());
    // Ensure the title is rendered.
    if (is_array($title)) {
      $title = $this->renderer->renderRoot($title);
    }

    $data = [
      'title' => $title,
      'content' => $content,
    ];

    $this->moduleHandler->alter('lupus_ce_renderer_response', $data, $bubbleable_metadata, $request, $route_match);

    $response = new CacheableJsonResponse();
    $response->setData($data);
    $response->addCacheableDependency($bubbleable_metadata);
    return $response;
  }

}
